@@ -6,6 +6,8 @@
  * others. All Rights Reserved.
  ********************************************************************/
 
+#include <stdbool.h>
+
 #include "cintltst.h"
 #include "unicode/ures.h"
 #include "unicode/ucurr.h"
@@ -68,9 +70,9 @@ isCurrencyPreEuro(const char* currencyKey){
         strcmp(currencyKey, "BEF") == 0 ||
         strcmp(currencyKey, "ITL") == 0 ||
         strcmp(currencyKey, "EEK") == 0){
-            return TRUE;
+            return true;
     }
-    return FALSE;
+    return false;
 }
 #if !UCONFIG_NO_FILE_IO && !UCONFIG_NO_LEGACY_CONVERSION
 static void
@@ -90,7 +92,7 @@ TestKeyInRootRecursive(UResourceBundle *root, const char *rootName,
         (void)currentBundleKey;    /* Suppress set but not used warning. */
         subBundle = ures_getNextResource(currentBundle, NULL, &errorCode);
         if (U_FAILURE(errorCode)) {
-            log_err("Can't open a resource for lnocale %s. Error: %s\n", locale, u_errorName(errorCode));
+            log_err("Can't open a resource for locale %s. Error: %s\n", locale, u_errorName(errorCode));
             continue;
         }
         subBundleKey = ures_getKey(subBundle);
@@ -121,7 +123,7 @@ TestKeyInRootRecursive(UResourceBundle *root, const char *rootName,
             int32_t minSize;
             int32_t subBundleSize;
             int32_t idx;
-            UBool sameArray = TRUE;
+            UBool sameArray = true;
             const int32_t *subRootBundleArr = ures_getIntVector(subRootBundle, &minSize, &errorCode);
             const int32_t *subBundleArr = ures_getIntVector(subBundle, &subBundleSize, &errorCode);
 
@@ -135,7 +137,7 @@ TestKeyInRootRecursive(UResourceBundle *root, const char *rootName,
 
             for (idx = 0; idx < minSize && sameArray; idx++) {
                 if (subRootBundleArr[idx] != subBundleArr[idx]) {
-                    sameArray = FALSE;
+                    sameArray = false;
                 }
                 if (strcmp(subBundleKey, "DateTimeElements") == 0
                     && (subBundleArr[idx] < 1 || 7 < subBundleArr[idx]))
@@ -170,14 +172,14 @@ TestKeyInRootRecursive(UResourceBundle *root, const char *rootName,
             else {
                 int32_t minSize = ures_getSize(subRootBundle);
                 int32_t idx;
-                UBool sameArray = TRUE;
+                UBool sameArray = true;
 
                 if (minSize > ures_getSize(subBundle)) {
                     minSize = ures_getSize(subBundle);
                 }
 
                 if ((subBundleKey == NULL
-                    || (subBundleKey != NULL &&  strcmp(subBundleKey, "LocaleScript") != 0 && !isCurrencyPreEuro(subBundleKey)))
+                    || (subBundleKey != NULL && strcmp(subBundleKey, "LocaleScript") != 0 && !isCurrencyPreEuro(subBundleKey)))
                     && ures_getSize(subRootBundle) != ures_getSize(subBundle))
                 {
                     log_err("Different size array with key \"%s\" in \"%s\" from root for locale \"%s\"\n"
@@ -203,7 +205,7 @@ TestKeyInRootRecursive(UResourceBundle *root, const char *rootName,
                     const UChar *localeStr = ures_getStringByIndex(subBundle,idx,&localeStrLen,&errorCode);
                     if (rootStr && localeStr && U_SUCCESS(errorCode)) {
                         if (u_strcmp(rootStr, localeStr) != 0) {
-                            sameArray = FALSE;
+                            sameArray = false;
                         }
                     }
                     else {
@@ -338,13 +340,15 @@ TestKeyInRootRecursive(UResourceBundle *root, const char *rootName,
                         subBundleKey,
                         ures_getKey(currentBundle),
                         locale);
-            } else if (string[0] == (UChar)0x20) {
+            /* foreignSpaceReplacement can be just a space */
+            } else if (string[0] == (UChar)0x20 && (strcmp(subBundleKey,"foreignSpaceReplacement"))) {
                 log_err("key \"%s\" in \"%s\" starts with a space in locale \"%s\"\n",
                         subBundleKey,
                         ures_getKey(currentBundle),
                         locale);
-            /* localeDisplayPattern/separator can end with a space */
-            } else if (string[len - 1] == (UChar)0x20 && (strcmp(subBundleKey,"separator"))) {
+            /* localeDisplayPattern/separator can end with a space, foreignSpaceReplacement can be just a space */
+            } else if (string[len - 1] == (UChar)0x20 && (strcmp(subBundleKey,"separator"))
+                    && (strcmp(subBundleKey,"foreignSpaceReplacement"))) {
                 log_err("key \"%s\" in \"%s\" ends with a space in locale \"%s\"\n",
                         subBundleKey,
                         ures_getKey(currentBundle),
@@ -418,7 +422,9 @@ TestKeyInRootRecursive(UResourceBundle *root, const char *rootName,
 #endif
         }
         else if (ures_getType(subBundle) == URES_TABLE) {
-            if (strcmp(subBundleKey, "availableFormats")!=0) {
+            if (strcmp(subBundleKey, "availableFormats")!=0 &&
+                strcmp(subBundleKey, "nameOrderLocales")!=0 &&
+                strcmp(subBundleKey, "namePattern")!=0 ) {
                 /* Here is one of the recursive parts */
                 TestKeyInRootRecursive(subRootBundle, rootName, subBundle, locale);
             }
@@ -820,7 +826,7 @@ findSetMatch( UScriptCode *scriptCodes, int32_t scriptsLen,
         uset_add(scripts[0], 0x2bc);
     }
     if(U_SUCCESS(status)){
-        UBool existsInScript = FALSE;
+        UBool existsInScript = false;
         /* iterate over the exemplarSet and ascertain if all
          * UChars in exemplarSet belong to the scripts returned
          * by getScript
@@ -839,15 +845,15 @@ findSetMatch( UScriptCode *scriptCodes, int32_t scriptsLen,
                 if(strCapacity == 0){
                     /* ok the item is a range */
                      for( j = 0; j < scriptsLen; j++){
-                        if(uset_containsRange(scripts[j], start, end) == TRUE){
-                            existsInScript = TRUE;
+                        if(uset_containsRange(scripts[j], start, end) == true){
+                            existsInScript = true;
                         }
                     }
-                    if(existsInScript == FALSE){
+                    if(existsInScript == false){
                         for( j = 0; j < scriptsLen; j++){
                             UChar toPattern[500]={'\0'};
                             char pat[500]={'\0'};
-                            int32_t len = uset_toPattern(scripts[j], toPattern, 500, TRUE, &status);
+                            int32_t len = uset_toPattern(scripts[j], toPattern, 500, true, &status);
                             len = myUCharsToChars(toPattern, pat, len);
                             log_err("uset_indexOf(\\u%04X)=%i uset_indexOf(\\u%04X)=%i\n", start, uset_indexOf(scripts[0], start), end, uset_indexOf(scripts[0], end));
                             if(len!=-1){
@@ -865,11 +871,11 @@ findSetMatch( UScriptCode *scriptCodes, int32_t scriptsLen,
                      * in the script set
                      */
                     for( j = 0; j < scriptsLen; j++){
-                        if(uset_containsString(scripts[j],str, strCapacity) == TRUE){
-                            existsInScript = TRUE;
+                        if(uset_containsString(scripts[j],str, strCapacity) == true){
+                            existsInScript = true;
                         }
                     }
-                    if(existsInScript == FALSE){
+                    if(existsInScript == false){
                         log_err("ExemplarCharacters and LocaleScript containment test failed for locale %s. \n", locale);
                     }
                 }
@@ -967,7 +973,7 @@ static void VerifyTranslation(void) {
                 log_err("error uloc_getDisplayLanguage returned %s\n", u_errorName(errorCode));
             }
             else {
-                strIdx = findStringSetMismatch(currLoc, langBuffer, langSize, mergedExemplarSet, FALSE, &badChar);
+                strIdx = findStringSetMismatch(currLoc, langBuffer, langSize, mergedExemplarSet, false, &badChar);
                 if (strIdx >= 0) {
                     log_err("getDisplayLanguage(%s) at index %d returned characters not in the exemplar characters: %04X.\n",
                         currLoc, strIdx, badChar);
@@ -1002,6 +1008,10 @@ static void VerifyTranslation(void) {
                         log_knownIssue("cldrbug:14995", "mai/sd_Deva day names use chars not in exemplars")) {
                     end = 0;
                 }
+                if (uprv_strncmp(currLoc,"ks_Deva",7) == 0 && 
+                        log_knownIssue("cldrbug:15355", "ks_Deva day names use chars not in exemplars")) {
+                    end = 0;
+                }
 
                 for (idx = 0; idx < end; idx++) {
                     const UChar *fromBundleStr = ures_getStringByIndex(resArray, idx, &langSize, &errorCode);
@@ -1009,7 +1019,7 @@ static void VerifyTranslation(void) {
                         log_err("error ures_getStringByIndex(%d) returned %s\n", idx, u_errorName(errorCode));
                         continue;
                     }
-                    strIdx = findStringSetMismatch(currLoc, fromBundleStr, langSize, mergedExemplarSet, TRUE, &badChar);
+                    strIdx = findStringSetMismatch(currLoc, fromBundleStr, langSize, mergedExemplarSet, true, &badChar);
                     if ( strIdx >= 0 ) { 
                         log_err("getDayNames(%s, %d) at index %d returned characters not in the exemplar characters: %04X.\n",
                             currLoc, idx, strIdx, badChar);
@@ -1035,6 +1045,10 @@ static void VerifyTranslation(void) {
                         log_knownIssue("cldrbug:14995", "sd_Deva month names use chars not in exemplars")) {
                     end = 0;
                 }
+                if (uprv_strncmp(currLoc,"ks_Deva",7) == 0 && 
+                        log_knownIssue("cldrbug:15355", "ks_Deva month names use chars not in exemplars")) {
+                    end = 0;
+                }
 
                 for (idx = 0; idx < end; idx++) {
                     const UChar *fromBundleStr = ures_getStringByIndex(resArray, idx, &langSize, &errorCode);
@@ -1042,7 +1056,7 @@ static void VerifyTranslation(void) {
                         log_err("error ures_getStringByIndex(%d) returned %s\n", idx, u_errorName(errorCode));
                         continue;
                     }
-                    strIdx = findStringSetMismatch(currLoc, fromBundleStr, langSize, mergedExemplarSet, TRUE, &badChar);
+                    strIdx = findStringSetMismatch(currLoc, fromBundleStr, langSize, mergedExemplarSet, true, &badChar);
                     if (strIdx >= 0) {
                         log_err("getMonthNames(%s, %d) at index %d returned characters not in the exemplar characters: %04X.\n",
                             currLoc, idx, strIdx, badChar);
@@ -1214,7 +1228,7 @@ static void TestExemplarSet(void){
             }
             if (!assertSuccess("uset_openPattern", &ec)) goto END;
 
-            existsInScript = FALSE;
+            existsInScript = false;
             itemCount = uset_getItemCount(exemplarSet);
             for (m=0; m<itemCount && !existsInScript; ++m) {
                 strLen = uset_getItem(exemplarSet, m, &start, &end, ubuf,
@@ -1224,21 +1238,21 @@ static void TestExemplarSet(void){
                 if (strLen == 0) {
                     for (j=0; j<codeLen; ++j) {
                         if (codeSets[j]!=NULL && uset_containsRange(codeSets[j], start, end)) {
-                            existsInScript = TRUE;
+                            existsInScript = true;
                             break;
                         }
                     }
                 } else {
                     for (j=0; j<codeLen; ++j) {
                         if (codeSets[j]!=NULL && uset_containsString(codeSets[j], ubuf, strLen)) {
-                            existsInScript = TRUE;
+                            existsInScript = true;
                             break;
                         }
                     }
                 }
             }
 
-            if (existsInScript == FALSE){
+            if (existsInScript == false){
                 log_err("ExemplarSet containment failed for locale : %s\n", locale);
             }
         }
@@ -1345,6 +1359,14 @@ static void TestCoverage(void){
         }
     }
 
+    // ICU-22149: Cover this code path even if the lang bundle is not present
+    UErrorCode localStatus = U_ZERO_ERROR;
+    UChar pattern[20];
+    ulocdata_getLocaleDisplayPattern(uld, pattern, 20, &localStatus);
+    if (U_FAILURE(localStatus) && localStatus != U_MISSING_RESOURCE_ERROR) {
+        log_err("ulocdata_getLocaleDisplayPattern coverage error %s", u_errorName(localStatus));
+    }
+
     sub = ulocdata_getNoSubstitute(uld);
     ulocdata_setNoSubstitute(uld,sub);
     ulocdata_close(uld);
@@ -1484,80 +1506,80 @@ static void TestAvailableIsoCodes(void){
 
     /* testing available codes with no time ranges */
     u_charsToUChars(eurCode, isoCode, (int32_t)uprv_strlen(usdCode) + 1);
-    if (ucurr_isAvailable(isoCode, U_DATE_MIN, U_DATE_MAX, &errorCode) == FALSE) {
+    if (ucurr_isAvailable(isoCode, U_DATE_MIN, U_DATE_MAX, &errorCode) == false) {
        log_data_err("FAIL: ISO code (%s) is not found.\n", eurCode);
     }
 
     u_charsToUChars(usdCode, isoCode, (int32_t)uprv_strlen(zzzCode) + 1);
-    if (ucurr_isAvailable(isoCode, U_DATE_MIN, U_DATE_MAX, &errorCode) == FALSE) {
+    if (ucurr_isAvailable(isoCode, U_DATE_MIN, U_DATE_MAX, &errorCode) == false) {
        log_data_err("FAIL: ISO code (%s) is not found.\n", usdCode);
     }
 
     u_charsToUChars(zzzCode, isoCode, (int32_t)uprv_strlen(zzzCode) + 1);
-    if (ucurr_isAvailable(isoCode, U_DATE_MIN, U_DATE_MAX, &errorCode) == TRUE) {
+    if (ucurr_isAvailable(isoCode, U_DATE_MIN, U_DATE_MAX, &errorCode) == true) {
        log_err("FAIL: ISO code (%s) is reported as available, but it doesn't exist.\n", zzzCode);
     }
 
     u_charsToUChars(lastCode, isoCode, (int32_t)uprv_strlen(zzzCode) + 1);
-    if (ucurr_isAvailable(isoCode, U_DATE_MIN, U_DATE_MAX, &errorCode) == FALSE) {
+    if (ucurr_isAvailable(isoCode, U_DATE_MIN, U_DATE_MAX, &errorCode) == false) {
        log_data_err("FAIL: ISO code (%s) is not found.\n", lastCode);
     }
 
     /* RHD was used from 1970-02-17  to 1980-04-18*/
     
     /* to = null */
-    if (ucurr_isAvailable(isoCode, date1970, U_DATE_MAX, &errorCode) == FALSE) {
+    if (ucurr_isAvailable(isoCode, date1970, U_DATE_MAX, &errorCode) == false) {
        log_data_err("FAIL: ISO code (%s) was available in time range >1970-01-01.\n", lastCode);
     }
 
-    if (ucurr_isAvailable(isoCode, date1975, U_DATE_MAX, &errorCode) == FALSE) {
+    if (ucurr_isAvailable(isoCode, date1975, U_DATE_MAX, &errorCode) == false) {
        log_data_err("FAIL: ISO code (%s) was available in time range >1975.\n", lastCode);
     }
 
-    if (ucurr_isAvailable(isoCode, date1981, U_DATE_MAX, &errorCode) == TRUE) {
+    if (ucurr_isAvailable(isoCode, date1981, U_DATE_MAX, &errorCode) == true) {
        log_err("FAIL: ISO code (%s) was not available in time range >1981.\n", lastCode);
     }
 
     /* from = null */
-    if (ucurr_isAvailable(isoCode, U_DATE_MIN, date1970, &errorCode) == TRUE) {
+    if (ucurr_isAvailable(isoCode, U_DATE_MIN, date1970, &errorCode) == true) {
        log_err("FAIL: ISO code (%s) was not available in time range <1970.\n", lastCode);
     }
 
-    if (ucurr_isAvailable(isoCode, U_DATE_MIN, date1975, &errorCode) == FALSE) {
+    if (ucurr_isAvailable(isoCode, U_DATE_MIN, date1975, &errorCode) == false) {
        log_data_err("FAIL: ISO code (%s) was available in time range <1975.\n", lastCode);
     }
 
-    if (ucurr_isAvailable(isoCode, U_DATE_MIN, date1981, &errorCode) == FALSE) {
+    if (ucurr_isAvailable(isoCode, U_DATE_MIN, date1981, &errorCode) == false) {
        log_data_err("FAIL: ISO code (%s) was available in time range <1981.\n", lastCode);
     }
 
     /* full ranges */
-    if (ucurr_isAvailable(isoCode, date1975, date1978, &errorCode) == FALSE) {
+    if (ucurr_isAvailable(isoCode, date1975, date1978, &errorCode) == false) {
        log_data_err("FAIL: ISO code (%s) was available in time range 1975-1978.\n", lastCode);
     }
 
-    if (ucurr_isAvailable(isoCode, date1970, date1975, &errorCode) == FALSE) {
+    if (ucurr_isAvailable(isoCode, date1970, date1975, &errorCode) == false) {
        log_data_err("FAIL: ISO code (%s) was available in time range 1970-1975.\n", lastCode);
     }
 
-    if (ucurr_isAvailable(isoCode, date1975, date1981, &errorCode) == FALSE) {
+    if (ucurr_isAvailable(isoCode, date1975, date1981, &errorCode) == false) {
        log_data_err("FAIL: ISO code (%s) was available in time range 1975-1981.\n", lastCode);
     }
 
-    if (ucurr_isAvailable(isoCode, date1970,  date1981, &errorCode) == FALSE) {
+    if (ucurr_isAvailable(isoCode, date1970,  date1981, &errorCode) == false) {
        log_data_err("FAIL: ISO code (%s) was available in time range 1970-1981.\n", lastCode);
     }
 
-    if (ucurr_isAvailable(isoCode, date1981,  date1992, &errorCode) == TRUE) {
+    if (ucurr_isAvailable(isoCode, date1981,  date1992, &errorCode) == true) {
        log_err("FAIL: ISO code (%s) was not available in time range 1981-1992.\n", lastCode);
     }
 
-    if (ucurr_isAvailable(isoCode, date1950,  date1970, &errorCode) == TRUE) {
+    if (ucurr_isAvailable(isoCode, date1950,  date1970, &errorCode) == true) {
        log_err("FAIL: ISO code (%s) was not available in time range 1950-1970.\n", lastCode);
     }
 
     /* wrong range - from > to*/
-    if (ucurr_isAvailable(isoCode, date1975,  date1970, &errorCode) == TRUE) {
+    if (ucurr_isAvailable(isoCode, date1975,  date1970, &errorCode) == true) {
        log_err("FAIL: Wrong range 1975-1970 for ISO code (%s) was not reported.\n", lastCode);
     } else if (errorCode != U_ILLEGAL_ARGUMENT_ERROR) {
        log_data_err("FAIL: Error code not reported for wrong range 1975-1970 for ISO code (%s).\n", lastCode);
